"""
Urls for porticus
"""
from django.conf.urls import url, patterns
from porticus.views import GalleryListView, GalleryDetailView, GalleryTreeView, AlbumDetailView, AlbumTagRessourcesView

urlpatterns = patterns('porticus.views',
    url(r'^$', GalleryListView.as_view(), name='porticus-galleries-index'),
    
    url(r'^(?P<gallery_slug>[-\w]+)/$', GalleryDetailView.as_view(), name='porticus-gallery-detail'),
    url(r'^tree/(?P<gallery_slug>[-\w]+)/$', GalleryTreeView.as_view(), name='porticus-gallery-tree'),
    
    url(r'^(?P<gallery_slug>[-\w]+)/(?P<album_slug>[-\w]+)/$', AlbumDetailView.as_view(), name='porticus-album-detail'),
    url(r'^(?P<gallery_slug>[-\w]+)/(?P<album_slug>[-\w]+)/tag/(?P<tag>[-\w\ ]+)/$', AlbumTagRessourcesView.as_view(), name='porticus-album-tag'),
)



"""Models of porticus.cmsplugins_album"""
from django.conf import settings
from django.db import models
from django.utils.translation import ugettext_lazy as _

from cms.models import CMSPlugin

from porticus.models import Gallery, Album

class GalleryPlugin(CMSPlugin):
    """CMS Plugin for displaying a Gallery"""

    gallery = models.ForeignKey(Album, verbose_name=_('album'), help_text=_('Gallery to display'))

    template_name = models.CharField(_('template'), max_length=100, help_text=_('Template used to render the plugin Album'), choices=settings.PORTICUS_GALLERY_PLUGIN_TEMPLATE_CHOICES, default=settings.PORTICUS_GALLERY_PLUGIN_TEMPLATE_DEFAULT, blank=False)

    def __unicode__(self):
        return self.gallery.name

class AlbumPlugin(CMSPlugin):
    """CMS Plugin for displaying a Album"""

    album = models.ForeignKey(Album, verbose_name=_('album'), help_text=_('Album to display'))

    template_name = models.CharField(_('template'), max_length=100, help_text=_('Template used to render the plugin Album'), choices=settings.PORTICUS_ALBUM_PLUGIN_TEMPLATE_CHOICES, default=settings.PORTICUS_ALBUM_PLUGIN_TEMPLATE_DEFAULT, blank=False)

    def __unicode__(self):
        return self.album.name



"""
DjangoCMS plugin for porticus
"""
from django.conf import settings
from django.utils.translation import ugettext_lazy as _

from cms.plugin_pool import plugin_pool
from cms.plugin_base import CMSPluginBase

from porticus.cmsplugin_porticus.models import GalleryPlugin as GalleryPluginModel
from porticus.cmsplugin_porticus.models import AlbumPlugin as AlbumPluginModel

class PorticusPluginBase(CMSPluginBase):
    module = 'Porticus'

class GalleryPlugin(PorticusPluginBase):
    """
    Standard plugin to embed a slideshow
    """
    model = GalleryPluginModel
    name = _('Gallery')
    render_template = settings.PORTICUS_GALLERY_PLUGIN_TEMPLATE_DEFAULT

    def render(self, context, instance, placeholder):
        self.render_template = instance.template_name
        
        context.update({
            'instance': instance,
            'gallery_instance': instance.gallery,
        })
        return context

class AlbumPlugin(PorticusPluginBase):
    """
    Standard plugin to embed a slideshow
    """
    model = AlbumPluginModel
    name = _('Album')
    fields = ('album', 'template_name')
    render_template = settings.PORTICUS_ALBUM_PLUGIN_TEMPLATE_DEFAULT

    def render(self, context, instance, placeholder):
        self.render_template = instance.template_name
        
        context.update({
            'instance': instance,
            'album_instance': instance.album,
            'gallery_instance': instance.album.gallery,
        })
        return context


plugin_pool.register_plugin(GalleryPlugin)
plugin_pool.register_plugin(AlbumPlugin)



"""Toolbar extensions for CMS"""
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _

from cms.toolbar_base import CMSToolbar
from cms.toolbar_pool import toolbar_pool


class PorticusToolbar(CMSToolbar):

    def populate(self):
        porticus_menu = self.toolbar.get_or_create_menu(
            'porticus-menu', _('Porticus'))

        url = reverse('admin:porticus_gallery_add')
        porticus_menu.add_sideframe_item(_('New gallery'), url=url)

        url = reverse('admin:porticus_album_add')
        porticus_menu.add_sideframe_item(_('New album'), url=url)

        url = reverse('admin:porticus_ressource_add')
        porticus_menu.add_sideframe_item(_('New ressource'), url=url)

        porticus_menu.add_break()

        url = reverse('admin:porticus_gallery_changelist')
        porticus_menu.add_sideframe_item(_('Galleries list'), url=url)

        url = reverse('admin:porticus_album_changelist')
        porticus_menu.add_sideframe_item(_('Albums list'), url=url)

        url = reverse('admin:porticus_ressource_changelist')
        porticus_menu.add_sideframe_item(_('Ressources list'), url=url)

        porticus_menu.add_break()

        url = reverse('admin:tagging_tag_changelist')
        porticus_menu.add_sideframe_item(_('Tags list'), url=url)


toolbar_pool.register(PorticusToolbar)



"""
Models for porticus
"""
import os

from django.conf import settings
from django.db import models
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _

from porticus.managers import RessourcePublishedManager, GalleryPublishedManager, AlbumPublishedManager

from mptt.models import MPTTModel, TreeForeignKey, TreeManager

from filer.fields.image import FilerImageField
from filer.fields.file import FilerFileField

from tagging.fields import TagField
from tagging.models import Tag

PUBLISHED_CHOICES = (
    (True, _('Published')),
    (False, _('Unpublished')),
)

class Gallery(models.Model):
    """Model representing a gallery"""

    name = models.CharField(_('name'), max_length=250)

    description = models.TextField(_('description'), blank=True)

    image = FilerImageField(verbose_name=_('image'), related_name="gallery_image", null=True, blank=True, default=None)

    template_name = models.CharField(_('template'), max_length=255, help_text=_('Template used to render the gallery'), choices=settings.PORTICUS_GALLERY_TEMPLATE_CHOICES, default=settings.PORTICUS_GALLERY_TEMPLATE_DEFAULT)

    publish = models.BooleanField(_('published'), choices=PUBLISHED_CHOICES, default=True)

    priority = models.IntegerField(_('display priority'), default=100)

    creation_date = models.DateTimeField(_('creation date'), auto_now_add=True)

    slug = models.SlugField(_('slug'), unique=True, max_length=100)

    objects = models.Manager()
    published = GalleryPublishedManager()

    #@models.permalink
    #def get_absolute_url(self):
        #return ('porticus-gallery-detail', (self.slug,))

    def get_tags(self):
        return Tag.objects.get_for_object(self)

    def __unicode__(self):
        return self.name

    class Meta:
        ordering = ('-priority', 'name')
        verbose_name = _('gallery')
        verbose_name_plural = _('galleries')


class Album(MPTTModel):
    """Model representing an album"""
    gallery = models.ForeignKey(Gallery)

    parent = TreeForeignKey('self', null=True, blank=True, related_name='children')

    name = models.CharField(_('name'), max_length=250)

    description = models.TextField(_('description'), blank=True)

    image = FilerImageField(verbose_name=_('image'), related_name="album_image", null=True, blank=True, default=None)

    template_name = models.CharField(_('template'), max_length=255, help_text=_('Template used to render the album'), choices=settings.PORTICUS_ALBUM_TEMPLATE_CHOICES, default=settings.PORTICUS_ALBUM_TEMPLATE_DEFAULT)

    publish = models.BooleanField(_('published'), choices=PUBLISHED_CHOICES, default=True)
    
    priority = models.IntegerField(_('display priority'), default=100)

    creation_date = models.DateTimeField(_('creation date'), auto_now_add=True)

    slug = models.SlugField(_('slug'), unique=True, max_length=100)

    def __unicode__(self):
        return self.name

    objects = TreeManager()
    published = AlbumPublishedManager()

    #@models.permalink
    #def get_absolute_url(self):
        #return ('porticus-album-detail', (self.gallery.slug, self.slug,))

    def get_tags(self):
        return Tag.objects.get_for_object(self)

    def get_published_children(self):
        """
        Return all ressources for the album and all its children
        """
        return self.get_children().filter(publish=True)
    
    def get_published_descendants(self):
        """
        Return all ressources for the album and its direct descendants
        """
        return self.get_descendants().filter(publish=True)
    
    def get_published_ressources(self):
        """
        Return all ressources for the album
        """
        return self.ressource_set.filter(publish=True).order_by('priority', 'name')

    class Meta:
        verbose_name = _('album')
        verbose_name_plural = _('albums')

    class MPTTMeta:
        order_insertion_by = ['gallery', 'priority', 'name']


class Ressource(models.Model):
    """Model for representing a ressource"""
    album = models.ForeignKey(Album)

    name = models.CharField(_('name'), max_length=250)

    description = models.TextField(_('description'), blank=True)

    image = FilerImageField(verbose_name=_('image'), related_name="ressource_image", null=True, blank=True, default=None)

    file_type = models.IntegerField(_('file type'), choices=settings.PORTICUS_RESSOURCE_FILETYPE_CHOICES, default=settings.PORTICUS_RESSOURCE_FILETYPE_DEFAULT)
    file = FilerFileField(verbose_name=_('file'), related_name="ressource_file", null=True, blank=True, default=None)
    file_url = models.URLField(_('file url'), blank=True)
    file_weight = models.CharField(_('file weight'), blank=True, max_length=15)

    publish = models.BooleanField(_('published'), choices=PUBLISHED_CHOICES, default=True)

    priority = models.IntegerField(_('display priority'), default=100)

    creation_date = models.DateTimeField(_('creation date'), auto_now_add=True)

    tags = TagField(_('tags'))

    objects = models.Manager()
    published = RessourcePublishedManager()

    def get_tags(self):
        return Tag.objects.get_for_object(self)

    @property
    def get_file(self):
        try:
            return self.file_url or self.file.url
        except ValueError:
            return None


    def clean(self):
        if not self.get_file:
            raise ValidationError(_('Please upload a file or give a file url'))

    def __unicode__(self):
        return self.name

    class Meta:
        ordering = ('-priority', 'name')
        verbose_name = _('ressource')
        verbose_name_plural = _('ressources')



# -*- coding: utf-8 -*-
"""
Common templates tags for porticus
"""
from django.conf import settings
from django import template
from django.utils.safestring import mark_safe
from django.shortcuts import get_object_or_404

from porticus.models import Gallery, Album

register = template.Library()

class GalleryList(template.Node):
    """
    Gallery list (without any pagination) as a HTML fragment
    """
    def __init__(self, template_varname=None, instance_varname=None):
        """
        :type insert_instance_varname: string or object ``django.db.models.Model``
        :param insert_instance_varname: Instance variable name or a string slug for 
                                        the current gallery
                                        
        :type template_varname: string
        :param template_varname: (optional) ...
        """
        self.template_varname = None
        if template_varname:
            self.template_varname = template.Variable(template_varname)
        self.instance_varname = None
        if instance_varname:
            self.instance_varname = template.Variable(instance_varname)
    
    def render(self, context):
        """
        :type context: object ``django.template.Context``
        :param context: Context tag object
        
        :rtype: string
        :return: the HTML for the list
        """
        # Default assume this is directly an instance
        current_gallery = None
        if self.instance_varname:
            current_gallery = self.instance_varname.resolve(context)
            if current_gallery:
                # Assume this is slug
                if isinstance(current_gallery, basestring):
                    current_gallery = Gallery.objects.get(slug=current_gallery, publish=True)
                # Only accept Gallery model instance
                elif not isinstance(current_gallery, Gallery):
                    raise template.TemplateSyntaxError, "You can only specify a Gallery instance or a slug"

        # Resolve optional template path
        template_path = settings.PORTICUS_GALLERIES_TEMPLATE_FRAGMENT
        if self.template_varname:
            try:
                resolved_var = self.template_varname.resolve(context)
            except template.VariableDoesNotExist:
                pass
            else:
                if resolved_var:
                    template_path = resolved_var
        
        gallery_list = Gallery.publish.all()
        
        subcontext = {
            'object_list': gallery_list,
            'current_gallery': current_gallery,
        }
        
        html = template.loader.get_template(template_path).render(template.Context(subcontext))
        
        return mark_safe(html)

@register.tag(name="porticus_gallery_list")
def do_porticus_gallery_list(parser, token):
    """
    Display the gallery list
    
    Usage : ::
    
        {% porticus_gallery_list [Optional template path] [Optional Gallery slug or instance] %}
    """
    args = token.split_contents()
    return GalleryList(*args[1:])

do_porticus_gallery_list.is_safe = True


class AlbumFragment(template.Node):
    """
    Album ressources as a HTML fragment
    """
    def __init__(self, instance_varname, template_varname=None):
        """
        :type insert_instance_varname: string or object ``django.db.models.Model``
        :param insert_instance_varname: Instance variable name or a string slug
                                        
        :type template_varname: string
        :param template_varname: (optional) ...
        """
        self.instance_varname = template.Variable(instance_varname)
        self.template_varname = None
        if template_varname:
            self.template_varname = template.Variable(template_varname)
    
    def render(self, context):
        """
        :type context: object ``django.template.Context``
        :param context: Context tag object
        
        :rtype: string
        :return: the HTML for the album
        """
        # Default assume this is directly an instance
        album_instance = self.instance_varname.resolve(context)
        # Assume this is slug
        if isinstance(album_instance, basestring):
            album_instance = Album.objects.get(slug=album_instance, publish=True)
        # Get the album's ressources
        ressources_list = album_instance.ressource_set.filter(publish=True)

        # Resolve optional template path
        template_path = settings.PORTICUS_ALBUM_TEMPLATE_FRAGMENT
        if self.template_varname:
            try:
                template_path = self.template_varname.resolve(context)
            except template.VariableDoesNotExist:
                pass
        
        subcontext = {
            'gallery_object': album_instance.gallery,
            'album_object': album_instance,
            'ressource_list': ressources_list,
        }
        
        html = template.loader.get_template(template_path).render(template.Context(subcontext))
        
        return mark_safe(html)

@register.tag(name="porticus_album_fragment")
def do_porticus_album_fragment(parser, token):
    """
    Display a album
    
    Usage : ::
    
        {% porticus_album_fragment [Optional Album slug or instance] [Optional template path] %}
    """
    args = token.split_contents()
    if len(args) < 2:
        raise template.TemplateSyntaxError, "You need to specify at less an \"Album\" instance or slug"
    else:
        return AlbumFragment(*args[1:])

do_porticus_album_fragment.is_safe = True

def porticus_album_list(gallery_instance, level=0):
    """
    Return a queryset list from a Gallery, this is a flat list of albums, not recursive
    
    Usage : ::
    
        {% porticus_album_list [Gallery slug or instance] [Optional level] %}
    """
    if isinstance(gallery_instance, basestring):
        return get_object_or_404(Gallery, slug=gallery_instance, publish=True).album_set.filter(level=0)
    
    return gallery_instance.album_set.filter(level=0)

register.assignment_tag(porticus_album_list)


# -*- coding: utf-8 -*-
"""
Sitemap entries

Only published entries are exposed.

* Album last modification datetime can be the album 'creation_date' or the 'creation_date' 
  value of its youngest Ressource if it is more recent that the album;
* Ressource that don't have an uploaded file are not exposed;

Be careful that exposing Ressources can turn into a very big 'sitemap.xml' file if you have many Ressources.
"""
import datetime

from django.contrib.sitemaps import Sitemap
from django.core.urlresolvers import reverse

from porticus import models

class PorticusGallerySitemap(Sitemap):
    changefreq = "monthly"
    priority = 1.0

    def items(self):
        return models.Gallery.published.order_by('priority', 'name')
    
    def location(self, obj):
        return reverse('porticus-gallery-detail', kwargs={'gallery_slug': obj.slug})
    
    def lastmod(self, obj):
        return obj.creation_date

class PorticusAlbumSitemap(Sitemap):
    changefreq = "monthly"
    priority = 0.5

    def items(self):
        return models.Album.objects.filter(publish=True).order_by('priority', 'name')
    
    def location(self, obj):
        return reverse('porticus-album-detail', kwargs={'gallery_slug': obj.gallery.slug, 'album_slug': obj.slug})
    
    def lastmod(self, obj):
        """
        Return the youngest datetime between "object creation date" and ressources "creation 
        dates", so if there are a more young ressource entry than the album, the 
        Album will exposes it 
        """
        return max([obj.creation_date]+list(obj.ressource_set.all().values_list('creation_date', flat=True)))

class PorticusRessourceSitemap(Sitemap):
    changefreq = "monthly"
    priority = 0.4

    def items(self):
        """
        Only exposes those ressources that have an uploaded file, excludes those ones 
        that have only a filled 'file_url'
        """
        return models.Ressource.published.exclude(file='').order_by('priority', 'name')
    
    def location(self, obj):
        return obj.file.url
    
    def lastmod(self, obj):
        return obj.creation_date



"""
Views for porticus
"""
from django.conf import settings
from django.shortcuts import get_object_or_404
from django.core.exceptions import ImproperlyConfigured

from django.views.generic.base import TemplateResponseMixin, View, TemplateView
from django.views.generic.list import BaseListView

from porticus.models import Gallery, Album, Ressource

from tagging.models import Tag, TaggedItem
from tagging.utils import calculate_cloud

class SimpleListView(TemplateResponseMixin, BaseListView):
    """
    Like generic.ListView but use only ``get_template`` to find template and not an
    automatic process on ``get_template_names``
    """
    pass


class AlbumConfinementMixin(object):
    """
    Mixin that include methods to confine a view that get objects from a specific Gallery+Album
    
    You still have to load objects yourself with method ``get_gallery_object`` and ``get_album_object`` 
    in your get/post/whatever view methods.
    """
    gallery_slug = None
    album_slug = None
    gallery_object = None
    album_object = None
    gallery_object = None

    def get_gallery_slug(self):
        return self.gallery_slug or self.kwargs.get('gallery_slug')

    def get_album_slug(self):
        return self.album_slug or self.kwargs.get('album_slug')

    def get_gallery_object(self):
        return get_object_or_404(Gallery, slug=self.get_gallery_slug(), publish=True)

    def get_album_object(self):
        return get_object_or_404(Album, slug=self.get_album_slug(), publish=True)

    def get_context_data(self, **kwargs):
        if self.album_object:
            kwargs['album_object'] = self.album_object
        if self.gallery_object:
            kwargs['gallery_object'] = self.gallery_object
        return super(AlbumConfinementMixin, self).get_context_data(**kwargs)


class GalleryListView(SimpleListView):
    """
    View to diplay a paginated list of galleries
    """
    paginate_by = settings.PORTICUS_GALLERIES_PAGINATION
    model = Gallery
    template_name = "porticus/gallery_list.html"


class GalleryDetailView(AlbumConfinementMixin, SimpleListView):
    model = Album
    paginate_by = settings.PORTICUS_ALBUMS_PAGINATION

    def get_queryset(self):
        return self.gallery_object.album_set.filter(publish=True).order_by('priority', 'name')
    
    def get_template_names(self):
        return (self.gallery_object.template_name,)

    def get(self, request, *args, **kwargs):
        self.gallery_object = self.get_gallery_object()
        return super(GalleryDetailView, self).get(request, *args, **kwargs)


class GalleryTreeView(GalleryDetailView):
    """
    Display the album tree for a gallery, inherit from GalleryDetailView, does not 
    follow the gallery template
    """
    paginate_by = None
    template_name = "porticus/gallery_tree.html"

    def get_queryset(self):
        return self.gallery_object.album_set.filter(publish=True)
    
    def get_template_names(self):
        return (self.template_name,)


class AlbumDetailView(AlbumConfinementMixin, SimpleListView):
    """
    Display the albums ressources and their tags
    """
    model = Ressource
    paginate_by = settings.PORTICUS_RESSOURCES_PAGINATION

    def get_queryset(self):
        return self.album_object.get_published_ressources()

    def get_template_names(self):
        return (self.album_object.template_name,)

    def get_context_data(self, **kwargs):
        kwargs = super(AlbumDetailView, self).get_context_data(**kwargs)
        
        tags_q = Tag.objects.usage_for_queryset(self.object_list, min_count=1)
        tags_q = calculate_cloud(tags_q, steps=6)
            
        kwargs.update({
            # Filter ressource tags from the ressource list queryset
            # Behavior to watch, maybe at this step, the queryset has been limited by the paginator
            'ressources_tags': tags_q,
        })
        return kwargs

    def get(self, request, *args, **kwargs):
        self.gallery_object = self.get_gallery_object()
        self.album_object = self.get_album_object()
        return super(AlbumDetailView, self).get(request, *args, **kwargs)

class AlbumTagRessourcesView(AlbumConfinementMixin, SimpleListView):
    """
    Template view to list tagged Ressources from an Album
    """
    model = Ressource
    paginate_by = settings.PORTICUS_RESSOURCES_PAGINATION
    template_name = 'porticus/tag_detail.html'
    tag = None

    def get_tag_object(self):
        tag = self.tag or self.kwargs.get('tag')
        return Tag.objects.get(name=tag)

    def get_ressources_queryset(self):
        if not hasattr(self, '_get_ressources_queryset_cache'):
            setattr(self, '_get_ressources_queryset_cache', self.album_object.get_published_ressources())
        return getattr(self, '_get_ressources_queryset_cache')

    def get_queryset(self):
        return TaggedItem.objects.get_by_model(self.get_ressources_queryset(), self.tag_object)

    def get_context_data(self, **kwargs):
        kwargs = super(AlbumTagRessourcesView, self).get_context_data(**kwargs)
        
        tags_q = Tag.objects.usage_for_queryset(self.get_ressources_queryset(), min_count=1)
        tags_q = calculate_cloud(tags_q, steps=6)
        
        kwargs.update({
            'tag_object': self.tag_object,
            'ressources_tags': tags_q,
        })
        return kwargs

    def get(self, request, *args, **kwargs):
        self.gallery_object = self.get_gallery_object()
        self.album_object = self.get_album_object()
        self.tag_object = self.get_tag_object()
        return super(AlbumTagRessourcesView, self).get(request, *args, **kwargs)
